searchData={"items":[{"type":"module","title":"quantile","doc":null,"ref":"quantile.html"},{"type":"function","title":"quantile.quantile/2","doc":null,"ref":"quantile.html#quantile/2"},{"type":"function","title":"quantile.rank_average/2","doc":null,"ref":"quantile.html#rank_average/2"},{"type":"type","title":"quantile.sample/0","doc":null,"ref":"quantile.html#t:sample/0"},{"type":"type","title":"quantile.samples/0","doc":null,"ref":"quantile.html#t:samples/0"},{"type":"module","title":"quantile_estimator","doc":null,"ref":"quantile_estimator.html"},{"type":"function","title":"quantile_estimator.compress/1","doc":null,"ref":"quantile_estimator.html#compress/1"},{"type":"function","title":"quantile_estimator.f_biased/1","doc":null,"ref":"quantile_estimator.html#f_biased/1"},{"type":"function","title":"quantile_estimator.f_targeted/1","doc":null,"ref":"quantile_estimator.html#f_targeted/1"},{"type":"function","title":"quantile_estimator.insert/2","doc":null,"ref":"quantile_estimator.html#insert/2"},{"type":"function","title":"quantile_estimator.inserts_since_compression/1","doc":null,"ref":"quantile_estimator.html#inserts_since_compression/1"},{"type":"function","title":"quantile_estimator.new/1","doc":null,"ref":"quantile_estimator.html#new/1"},{"type":"function","title":"quantile_estimator.quantile/2","doc":null,"ref":"quantile_estimator.html#quantile/2"},{"type":"type","title":"quantile_estimator.data_sample/0","doc":null,"ref":"quantile_estimator.html#t:data_sample/0"},{"type":"type","title":"quantile_estimator.invariant/0","doc":null,"ref":"quantile_estimator.html#t:invariant/0"},{"type":"extras","title":"Overview","doc":"# Quantile estimator written in Erlang\n\nThis library provides a memory-efficient data structure to store distributions of larges sets of data.","ref":"readme.html"},{"type":"extras","title":"Source - Overview","doc":"This implementation is based on:\n\nCormode et. al.:\n\"Effective Computation of Biased Quantiles over Data Streams\"\n\nAbstract:\n\n\"Skew is prevalent in many data sources such as IP traffic streams. To continually summarize the distribution of such data, a high-biased set of quantiles (e.g., 50th, 90th and 99th percentiles) with finer error guarantees at higher ranks (e.g., errors of 5, 1 and 0.1 percent, respectively) is more useful than uniformly distributed quantiles (e.g., 25th, 50th and 75th percentiles) with uniform error guarantees. In this paper, we address the following two problems. First, can we compute quantiles with finer error guarantees for the higher ranks of the data distribution effectively, using less space and computation time than computing all quantiles uniformly at the finest error? Second, if specific quantiles and their error bounds are requested a priori, can the necessary space usage and computation time be reduced? We answer both questions in the affirmative by formalizing them as the “high-biased ” and the “targeted ” quantiles problems, respectively, and presenting algorithms with provable guarantees, that perform significantly better than previously known solutions for these problems. We implemented our algorithms in the Gigascope data stream management system, and evaluated alternate approaches for maintaining the relevant summary structures. Our experimental results on real and synthetic IP data streams complement our theoretical analyses, and highlight the importance of lightweight, non-blocking implementations when maintaining summary structures over high-speed data streams.\"\n\nThe full text publication is included in the `doc/cormode_et_al.pdf` (with permission of the author).","ref":"readme.html#source"},{"type":"extras","title":"Installation - Overview","doc":"Building:\n```\ngit clone git://github.com/odo/quantile_estimator.git\ncd quantile_estimator/\nmake\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"```erlang\nplattfisch:quantile_estimator odo$ make shell\nerl -pz ebin deps/*/ebin\nErlang R16A (erts-5.10) [source] [smp:4:4] [async-threads:10] [hipe] [kernel-poll:false]\n\nEshell V5.10  (abort with ^G)\n1> IV = quantile_estimator:f_targeted([{0.05, 0.005}, {0.5, 0.02}, {0.95, 0.005}]).\n#Fun \n```\nSo first we generated a function specifying that we want a guarantee that the 5th, 50th and 95th percentile are preserved with an error no more than 0.5%, 2% and 0.5 %, respectively. Please note the errors specified are errors in rank, the error in value might vary according to the shape of your distribution.\n\nNow we take a new empty estimator and fill it with 10 000 random samples, giving a uniform distribution.\n\n```erlang\n2> QE = quantile_estimator:new(IV).\n{quantile_estimator,0,0,0,[], #Fun }\n3> QERandom = lists:foldl(\n    fun(_, QE) -> quantile_estimator:insert(random:uniform(), QE) end\n    , QE\n    , lists:seq(1, 10000)).\n[...]\n4> {Q005, Q05, Q095} = {\n    quantile_estimator:quantile(0.05, QERandom)\n    , quantile_estimator:quantile(0.5, QERandom)\n    , quantile_estimator:quantile(0.95, QERandom)\n}.\n{0.04771803620573256,0.4810684987033724,0.9473718796803361}\n```\n\nInserting and compressing are O(n) so it is a good idea to compress from time to time.\nWhen the data is uncompressed as in the example, with successive compression passes the size will quickly converge to the minimum (here 1/120 of the original size after 11 passes).\n\n```erlang\n5> erts_debug:size(QERandom).\n110066\n6> {Sizes, QERandomCompressed} = lists:foldl(\n    fun(_, {Sizes, QE}) -> QEC = quantile_estimator:compress(QE), {[{erts_debug:size(QEC)}|Sizes], QEC} end\n    , {[], QERandom}\n    , lists:seq(1, 20)\n).\n{[{902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {902},\n  {913},\n  {946},\n  {1089},\n  {1430},\n  {2222},\n  {3861},\n  {7216},\n  {14036},\n  {27698},\n  {55121}],\n {quantile_estimator,10000,76,0,\n                     [{group,2.4128016570301725e-4,1,0,1},\n                      {group,0.007869355524596777,67,20,2},\n                      {group,0.012289777300297455,48,8,69},\n                      {group,0.02016443337909468,68,19,117},\n                      {group,0.027114225047992102,64,28,185},\n                      {group,0.0324600651775544,56,33,249},\n                      {group,0.0351798560323231,32,61,305},\n                      {group,0.04220816673938166,68,17,337},\n                      {group,0.048274226780547824,80,0,405},\n                      {group,0.05430006206638027,62,0,485},\n                      {group,0.06088946003756135,64,33,547},\n                      {group,0.06343376254881306,32,54,611},\n                      {group,0.06661410852498184,32,68,643},\n                      {group,0.07409963150685184,79,38,675},\n                      {group,0.07811247195447457,52,98,754},\n                      {group,0.08519015363469817,80,27,806},\n                      {group,0.09009917743705032,64,101,886},\n                      {group,0.09858046185837788,90,80,...},\n                      {group,0.11388492012352502,145,...},\n                      {group,0.12492433355564558,...},\n                      {group,...},\n                      {...}|...],\n                     #Fun }}\n7> {Q005C, Q05C, Q095C} = {\n    quantile_estimator:quantile(0.05, QERandomCompressed)\n    , quantile_estimator:quantile(0.5, QERandomCompressed)\n    , quantile_estimator:quantile(0.95, QERandomCompressed)\n}.\n{0.05430006206638027,0.49830876273384694,0.9514831018482353}\n```\n\nWe can confirm that the maximum error we are expecting is probably not exceeded.\n\n```erlang\n8> {abs(Q005 - Q005C), abs(Q05 - Q05C), abs(Q095 - Q095C)}.\n{\n  0.006582025860647711\n  , 0.017240264030474517\n  , 0.0041112221678991645\n}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Real world data - Overview","doc":"The test data set is the population of 14593 settlements in the US which is a Pareto distribution (\"long tail\"). Please find the example data in the `private` directory.\n\n# Performance\n\nWhen inserting the test data (14593 numbers) and compressing every 10 inserts the rate of inserts is around 41 000 inserts/s.\n\n# Compression rate\n\nWhen compressing every 10 inserts, the size of the data structure converges to around 60 elements / bins.\n\n![compression rate](https://raw.github.com/odo/quantile_estimator/master/doc/compression.png \"compression rate\")\n\n# Accuracy\n\nValues can be represented with high accuracy while maintaining high compression rates.\n\n![accuracy](https://raw.github.com/odo/quantile_estimator/master/doc/accuracy.png \"accuracy\")","ref":"readme.html#real-world-data"},{"type":"extras","title":"Tests: - Overview","doc":"eunit:\n```\nmake test\n```\ndialyzer:\n```\nmake check\n```","ref":"readme.html#tests"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2021 Florian Odronitz\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}